#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses
import events
import help
import config
import window

def getitems(section):
    items = []
    if section:
        for function in config.keybindings[section].keys():
            keys = list(config.keybindings[section][function])
            keys.sort()
            keynames = []
            for key in keys:
                keynames += [help.keynames[key]]
            descr = help.descriptions[section][function][1]
            items += [(keynames, descr)]

    return items


class helpwin(window.window):

    def showitems(self):
        y = self.iy
        for item in self.items[self.first:]:
            self.win.addstr(y, 1, " "*self.iw, self.colors["background"])
            self.win.move(y, 1)
            for keyname in item[0][:-1]:
                self.win.addstr(keyname, self.colors["key"])
                self.win.addstr("/", self.colors["description"])
            self.win.addstr(item[0][-1], self.colors["key"])
            assert type(item[1])==type(""), item[1]
            self.win.addnstr(y, 35, item[1], self.iw-35)
            y += 1
            if y>=self.iy+self.ih:
                break

    def __init__(self, screen, maxh, maxw, colors, channel, timer):
        self.maxh = maxh
        self.maxw = maxw
        self.channel = channel
        self.timer = timer
        
        # for identification purposes, we only generate this once
        self.hidewindowevent = events.hidewindow(self)

        window.window.__init__(self,
                               screen, 1, 1, 0, 0,
                               colors,
                               "PyTone Help", 0)

        self.channel.subscribe(events.hidewindow, self.hidewindow)
        self.channel.subscribe(events.keypressed, self.keypressed)
        self.channel.subscribe(events.mouseevent, self.mouseevent)
        self.channel.subscribe(events.focuschanged, self.focuschanged)

    def resize(self, maxh, maxw):
        self.maxh = maxh
        self.maxw = maxw

    def showhelp(self, context):
        self.items = getitems("global")+getitems(context)
        self.items.sort()

        height = min(len(self.items)+2, self.maxh-2)
        self.first = 0
        if self.maxw<=80:
            width = self.maxw
        else:
            width = 80 + int((self.maxw-80)*0.5)
        y = max(0, (self.maxh-height)/2)
        x = max(0, (self.maxw-width)/2)

        window.window.resize(self, height, width, y, x)
        self.update()
        self.timer.sendeventin(self.hidewindowevent, 5, replace=1)

    # event handler

    def keypressed(self, event):
        if self.hasfocus():
            key = event.key
            # XXX: do we need own keybindings for help?
            if key in config.keybindings["filelist"]["selectnext"]:
                self.first = min(len(self.items)-self.ih, self.first+1)
            elif key in config.keybindings["filelist"]["selectprev"]:
                self.first = max(0, self.first-1)
            elif key in config.keybindings["filelist"]["selectnextpage"]:
                self.first = min(len(self.items)-self.ih, self.first+self.ih)
            elif key in config.keybindings["filelist"]["selectprevpage"]:
                self.first = max(0, self.first-self.iw)
            elif key in config.keybindings["filelist"]["selectfirst"]:
                self.first = 0
            elif key in config.keybindings["filelist"]["selectlast"]:
                self.first = len(self.items)-self.ih
            else:
                self.hide()
                raise events.TerminateEventProcessing
                return

            self.update()
            self.timer.sendeventin(self.hidewindowevent, 5, replace=1)
        
            raise events.TerminateEventProcessing

    def mouseevent(self, event):
        if self.hasfocus():
            self.hide()
            raise events.TerminateEventProcessing

    def focuschanged(self, event):
        # we either have focus, or we disappear...
        if not self.hasfocus():
            self.hide()



    def update(self):
        self.top()
        window.window.update(self)
        self.showitems()
        
        

